<h2><center>Shell 变量</center></h2>

------

## 一：概述

### 1. 介绍

​	变量可以通过变量名访问，在指令式语言中，变量通常是可变的；在某些条件下也是不可变的。

### 2. 变量规则

- 命名只能使用英文字母，数字和下划线，首个字符不能以数字开头
- 中间不能有空格，可以使用下划线
- 不能使用标点符号
- 不能使用bash里的关键字

## 二：变量分类

### 1. 自定义变量

- 定义变量：`变量名=变量值`
- 引用变量：`$变量名` 或 `${变量名}`
- 查看变量：`echo $变量名`
- 取消变量：`unset 变量名`
- 作用范围：仅在当前shell中有效

### 2. 环境变量

- 定义环境变量：
  1. `export back_dir2=/home/backup`
  2. `export back_dir1` 将自定义变量转换成环境变量
- 引用环境变量：`$ 变量名` 或 `${变量名}`
- 查看环境变量：`echo $变量名`
- 取消环境变量：`unset 变量名`
- 变量作用范围：在当前shell和子shell有效

注意：

- 环境变量拥有可继承性：`export`之后就拥有继承性。
- 永久生效：写到环境变量脚本，`/etc/profile` `~/.baserc` `~/.bash_profile` `/etc/bashrc`

### 3. 位置变量

```shell
$1 $2 $3 $4 $5 $6 $7 $8 $9 $(10)
```

**案例：**

```bash
[root@wxin ~]# vim test.sh
#!/bin/bash
echo "$1"

[root@wxin ~]# bash test.sh 123
123
```

### 4. 预定义变量

```shell
$0       # 脚本名
$*		# 所有的参数
$@ 	    # 所有的参数
$# 	    # 参数的个数
$$ 	    # 当前进程的PID
$!      # 上一个后台进程的PID
$?	    # 上一个命令的返回值 0表示成功
```

**案例：**

```bash
[root@wxin ~]# vim test.sh
echo "第2个位置参数是$2"
echo "第1个位置参数是$1"
echo "第4个位置参数是$4"

echo "所有参数是: $*"
echo "所有参数是: $@"
echo "参数的个数是: $#"
echo "当前进程的PID是: $$"

echo '$1='$1
echo '$2='$2
echo '$3='$3
echo '$*='$*
echo '$@='$@
echo '$#='$#
echo '$$='$$
```

**$*和$@区别**

- 当$*和$@没有被引用的时候，它们确实没有什么区别，都会把位置参数当成一个个体。最好不要使用，因为如果位置参数中带有空格或者通配符的情况下，可能结果会和想要的不一样
- "$\*"   会把所有位置参数当成一个整体（或者说当成一个字符串），如果没有位置参数，则"$\*"为空，如果有两个位置参数并且IFS为空格时，"$*"相当于"$1 $2"
- "$@" 会把所有位置参数当成一个单独的字段，如果没有位置参数（$#为0），则"$@"展开为空（不是空字符串，而是空列表），如果存在一个位置参数，则"$@"相当于"$1"，如果有两个参数，则"$@"相当于"$1" "$2"等等

```bash
[root@wxin ~]# vim a.sh
#!/bin/bash
for i in $@
do
        echo $i
done

for i in $*
do
        echo $i
done

for i in "$@"
do
        echo $i
done

for i in "$*"
do
        echo $i
done
```

## 三：变量赋值

### 1. 显示赋值

​	变量名=变量值

**示例：**

```bash
[root@wxin ~]# ip1=192.168.1.251
[root@wxin ~]# school="BeiJing 1000phone"
[root@wxin ~]# today1=`date +%F`
[root@wxin ~]# today2=$(date +%F)
```

### 2. 键盘输入

**语法格式：**

```shell
read 变量名
read -p "提示信息：" 变量名
read -t 5 -p "提示信息：" 变量名  # -t 后面跟秒数，定义输入字符的等待时间
read -n 2 变量名                 # -n 后跟一个数字，定义输入文本的长度，很实用
read -s -p "" 变量名             # -s 是隐藏用户输入
read [-ers][-a 数组][-d 分隔符][-i 缓冲区文字][-n 读取字符数][-N 读取字符数][-p 提示符][-t 超时][-u 文件描述符][名称...]
```

**示例：**

```bash
[root@wxin ~]# vim first.sh
back_dir1=/var/backup
read -p "请输入你的备份目录: " back_dir2
echo $back_dir1
echo $back_dir2
[root@wxin ~]# sh first.sh


[root@wxin ~]# vim ping2.sh
#!/bin/bash							
read -p "Input IP: " ip                
ping -c2 $ip &>/dev/null		
if [ $? = 0 ];then					
        		echo "host $ip is ok"	    
else											
        		echo "host $ip is fail"	    
fi											
[root@wxin ~]# chmod a+x ping2.sh
[root@wxin ~]# ./ping.sh
```

### 4. 引用变量

`""`：弱引用 可以实现变量和命令的替换

`''`：强引用 不完成变量替换

反引：命令替换 等价于 $()   反引号中的shell命令会被先执行

**示例：**

```bash
[root@wxin ~]# school=1000phone
# 强引用
[root@wxin ~]# echo "${school} is good"
1000phone is good
# 弱引用
[root@wxin ~]# echo '${school} is good'
${school} is good
# 反引
[root@wxin ~]# touch `date +%F`_file1.txt  
[root@wxin ~]# touch $(date +%F)_file2.txt 
[root@wxin ~]# disk_free3="df -Ph |grep '/$' |awk '{print $4}'"	# 错误
[root@wxin ~]# disk_free4=$(df -Ph |grep '/$' |awk '{print $4}')
[root@wxin ~]# disk_free5=`df -Ph |grep '/$' |awk '{print $4}'`
```

## 四：变量运算

### 1. 整数运算

方法一：`expr`

```bash
[root@wxin ~]# expr 1 + 2
[root@wxin ~]# expr $num1 + $num2    			+  -  \*  /  %
```

方法二：`$(())`

```bash
[root@wxin ~]# echo $(($num1+$num2))      	+  -  *  /   %
[root@wxin ~]# echo $((num1+num2))
[root@wxin ~]# echo $((5-3*2))	 
[root@wxin ~]# echo $(((5-3)*2))
[root@wxin ~]# echo $((2**3))
[root@wxin ~]# sum=$((1+2)); echo $sum
```

方法三：`$[]`

```bash
[root@wxin ~]# echo $[5+2]			+  -  *  /  %
[root@wxin ~]# echo $[5**2]
```

方法四：`let`

```bash
[root@wxin ~]# let sum=2+3; echo $sum
[root@wxin ~]# let i++; echo $i
```

### 2. 小数运算

​	使用bc做小数运算，scale指定小数点位数

**加法运算(scale参数无效)**

```bash
[root@wxin ~]# echo "5.999 + 5.001"|bc
6.000
[root@wxin ~]# echo "5.111+ 5.1114"|bc
10.2224
```

**减法运算(scale参数无效)**

```bash
[root@wxin ~]# echo "2.22 - 1.11"|bc
1.11
```

**乘法运算**

```bash
[root@wxin ~]# echo "5.12 * 5.6000"|bc
28.6720
```

注意：

​	乘积小数点位数默认以乘数中小数点位数最多的为准（不指定scale参数）

**除法运算**

```bash
[root@wxin ~]# echo "scale=2;9.898 / 1.11"|bc
8.91
[root@wxin ~]# echo "9.898 / 1.11"|bc
8
```

## 五：变量截取

### 1. 匹配截取

```bash
[root@wxin ~]# url=www.sina.com.cn  
[root@wxin ~]# echo ${#url}			  # 获取变量值的长度
15
[root@wxin ~]# echo ${url}			      # 标准查看
www.sina.com.cn
[root@wxin ~]# echo ${url#*.}		      # 从前往后，最短匹配
sina.com.cn
[root@wxin ~]# echo ${url##*.}		      # 从前往后，最长匹配	贪婪匹配
cn
[root@wxin ~]# url=www.sina.com.cn
[root@wxin ~]# echo ${url}
www.sina.com.cn
[root@wxin ~]# echo ${url%.*}		      # 从后往前，最短匹配
www.sina.com
[root@wxin ~]# echo ${url%%.*}	          # 从后往前，最长匹配	贪婪匹配
www

[root@wxin ~]# url=www.sina.com.cn
[root@wxin ~]# echo ${url#a.}
www.sina.com.cn
[root@wxin ~]# echo ${url#*sina.}
com.cn

[root@wxin ~]# echo $HOSTNAME
wxin.1000phone.com
[root@wxin ~]# echo ${HOSTNAME%%.*}
wxin
```

### 2. 索引切片

```bash
[root@wxin ~]# echo ${url:0:5}
[root@wxin ~]# echo ${url:5:5}		
[root@wxin ~]# echo ${url:5}
```

### 3. 字符替换

```bash
[root@wxin ~]# url=www.sina.com.cn
[root@wxin ~]# echo ${url/sina/baidu}
www.baidu.com.cn

[root@wxin ~]# url=www.sina.com.cn
[root@wxin ~]# echo ${url/n/N}
www.siNa.com.cn
[root@wxin ~]# echo ${url//n/N}		#贪婪匹配
www.siNa.com.cN
```

### 4. 自增运算

对变量的值的影响

```bash
[root@wxin ~]# i=1
[root@wxin ~]# let i++
[root@wxin ~]# echo $i
2
[root@wxin ~]# j=1
[root@wxin ~]# let ++j
[root@wxin ~]# echo $j
2
```

对表达式的值的影响

```bash
[root@wxin ~]# unset i
[root@wxin ~]# unset j

[root@wxin ~]# i=1
[root@wxin ~]# j=1

[root@wxin ~]# let x=i++         先赋值，再运算
[root@wxin ~]# let y=++j         先运算，再赋值

[root@wxin ~]# echo $i
2
[root@wxin ~]# echo $j
2

[root@wxin ~]# echo $x
1
[root@wxin ~]# echo $y
2
```

### 5. `${变量名-新变量值}`

```bash
[root@wxin ~]# unset var1
[root@wxin ~]# echo ${var1}
[root@wxin ~]# echo ${var1-aaaaa}
aaaaa

[root@wxin ~]# var2=111
[root@wxin ~]# echo ${var2-bbbbb}
111

[root@wxin ~]# var3=
[root@wxin ~]# echo ${var3-ccccc}
```

变量没有被赋值：会使用"新的变量值"替代

变量有被赋值（包括空值）： 不会被替代

```bash
[root@wxin ~]# unset var1
[root@wxin ~]# unset var2
[root@wxin ~]# unset var3

[root@wxin ~]# var2=
[root@wxin ~]# var3=111
[root@wxin ~]# echo ${var1:-aaaa}
aaaa
[root@wxin ~]# echo ${var2:-aaaa}
aaaa
[root@wxin ~]# echo ${var3:-aaaa}
111
```

### 6. `${变量名:-新变量值}`

变量没有被赋值（包括空值）：都会使用“新的变量值“ 替代

变量有被赋值： 不会被替代

```bash
[root@wxin ~]# echo ${var3+aaaa}
[root@wxin ~]# echo ${var3:+aaaa}

[root@wxin ~]# echo ${var3=aaaa}
[root@wxin ~]# echo ${var3:=aaaa}

[root@wxin ~]# echo ${var3?aaaa}
[root@wxin ~]# echo ${var3:?aaaa}
```